const guildNameRestrictions: genericValidation = {
	length: { min: 3, max: 100 },
	regex: /^[\p{L}\p{N} ._-]+$/u,
};

function isValidGuildName(rawName: string): validationResult {
	const name = rawName.trim().normalize("NFC");

	if (!name) return { valid: false, error: "Guild name is required" };

	if (name.length < guildNameRestrictions.length.min)
		return { valid: false, error: "Guild name is too short" };

	if (name.length > guildNameRestrictions.length.max)
		return { valid: false, error: "Guild name is too long" };

	if (!guildNameRestrictions.regex.test(name))
		return { valid: false, error: "Guild name contains invalid characters" };

	if (/^[._ -]|[._ -]$/.test(name))
		return {
			valid: false,
			error: "Guild name can't start or end with special characters",
		};

	return { valid: true, name };
}

const channelNameRestrictions: genericValidation = {
	length: { min: 1, max: 100 },
	regex: /^[\p{L}\p{N}\p{Emoji}_\-.]+$/u, // emojis + underscore, dot, dash ?
};

function isValidChannelName(rawName: string): validationResult {
	const name = rawName.trim().normalize("NFC");

	if (!name) return { valid: false, error: "Channel name is required" };

	if (name.length < channelNameRestrictions.length.min)
		return { valid: false, error: "Channel name is too short" };

	if (name.length > channelNameRestrictions.length.max)
		return { valid: false, error: "Channel name is too long" };

	if (/\s/.test(name))
		return { valid: false, error: "Channel name cannot contain spaces" };

	if (!channelNameRestrictions.regex.test(name))
		return {
			valid: false,
			error: "Channel name contains invalid characters",
		};

	if (/^[._-]|[._-]$/.test(name))
		return {
			valid: false,
			error: "Channel name can't start or end with special characters",
		};

	return { valid: true, name };
}

const categoryNameRestrictions: genericValidation = {
	length: { min: 1, max: 100 },
	regex: /^[\p{L}\p{N} ._-]+$/u,
};

function isValidCategoryName(rawName: string): validationResult {
	const name = rawName.trim().normalize("NFC");

	if (!name) return { valid: false, error: "Category name is required" };

	if (name.length < categoryNameRestrictions.length.min)
		return { valid: false, error: "Category name is too short" };

	if (name.length > categoryNameRestrictions.length.max)
		return { valid: false, error: "Category name is too long" };

	if (!categoryNameRestrictions.regex.test(name))
		return {
			valid: false,
			error: "Category name contains invalid characters",
		};

	if (/^[._ -]|[._ -]$/.test(name))
		return {
			valid: false,
			error: "Category name can't start or end with special characters",
		};

	return { valid: true, name };
}

export { isValidGuildName, isValidChannelName, isValidCategoryName };
