import { logger } from "@creations.works/logger";
import { cassandra } from "@lib/cassandra";
import { jsonResponse } from "@lib/http";
import { pika } from "@lib/pika";
import { isValidChannelName } from "@lib/validators";
import type { Client } from "cassandra-driver";

const routeDef: RouteDef = {
	method: "POST",
	accepts: "application/json",
	returns: "application/json;charset=utf-8",
	needsBody: "json",
};

async function handler(
	request: ExtendedRequest,
	requestBody: unknown,
): Promise<Response> {
	const user: UserSession | null = request.session;
	if (!user) {
		return jsonResponse(401, {
			message: "Unauthorized",
			error: "You must be logged in to create a channel",
		});
	}

	const { name, type, topic, position, category_id } = requestBody as Record<
		string,
		unknown
	>;

	const { id: guildId } = request.params;
	if (!guildId || typeof guildId !== "string") {
		return jsonResponse(400, {
			message: "Missing or invalid guild ID",
		});
	}

	const errors: string[] = [];

	if (typeof name !== "string") {
		errors.push("Channel name is required");
	} else {
		const result = isValidChannelName(name);
		if (!result.valid) errors.push(result.error ?? "Invalid channel name");
	}

	if (type !== "text" && type !== "voice") {
		errors.push("Invalid or missing type (must be 'text' or 'voice')");
	}

	if (
		topic !== undefined &&
		(typeof topic !== "string" || topic.length > 1024)
	) {
		errors.push("Invalid topic");
	}

	if (
		position !== undefined &&
		(typeof position !== "number" || position < 0)
	) {
		errors.push("Invalid position");
	}

	if (category_id !== undefined) {
		if (typeof category_id !== "string") {
			errors.push("Invalid category_id");
		} else {
			const result = pika.validate(category_id, "category");
			if (!result) errors.push("Invalid category name format");
		}
	}

	if (errors.length > 0) {
		return jsonResponse(400, {
			message: "Validation failed",
			error: errors.join("; "),
		});
	}

	const now = new Date();
	const channelId = pika.gen("channel");
	const client: Client = cassandra.getClient();

	let finalPosition = typeof position === "number" ? position : 0;

	if (position === undefined) {
		try {
			const result = await client.execute(
				"SELECT position FROM channels WHERE guild_id = ?",
				[guildId],
				{ prepare: true },
			);

			let maxPos = -1;
			for (const row of result.rows) {
				if (typeof row.position === "number" && row.position > maxPos) {
					maxPos = row.position;
				}
			}

			finalPosition = maxPos + 1;
		} catch (err) {
			logger.error(["Failed to fetch existing channels:", err as Error]);
			return jsonResponse(500, {
				message: "Internal server error",
				error: "Could not determine channel position",
			});
		}
	}

	if (category_id !== undefined) {
		try {
			const categoryCheck = await client.execute(
				"SELECT id FROM categories WHERE id = ? AND guild_id = ?",
				[category_id, guildId],
				{ prepare: true },
			);

			if (categoryCheck.rowLength === 0) {
				return jsonResponse(400, {
					message: "Validation failed",
					error: "Provided category does not exist in this guild",
				});
			}
		} catch (err) {
			logger.error(["Failed to verify category:", err as Error]);
			return jsonResponse(500, {
				message: "Internal server error",
				error: "Failed to verify category existence",
			});
		}
	}

	// TODO: Check if the user has permission to create a channel in this guild

	try {
		await client.execute(
			`INSERT INTO channels (
				id, guild_id, name, type, topic, position, category_id, created_at, updated_at
			) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)`,
			[
				channelId,
				guildId,
				name,
				type,
				topic ?? null,
				finalPosition,
				category_id ?? null,
				now,
				now,
			],
			{ prepare: true },
		);

		logger.custom(
			"[CHANNEL CREATE]",
			`(${channelId})`,
			`${name} in guild ${guildId} by ${user.id}`,
			"36",
		);

		return jsonResponse(201, {
			message: "Channel created successfully",
			data: {
				id: channelId,
				guildId,
				name,
				type,
				topic: topic ?? null,
				position: finalPosition,
				category_id: category_id ?? null,
				created_at: now,
				updated_at: now,
			},
		});
	} catch (error) {
		logger.error(["Failed to create channel:", error as Error]);
		return jsonResponse(500, {
			message: "Internal server error",
			error: "Failed to create channel",
		});
	}
}

export { handler, routeDef };
