import { logger } from "@creations.works/logger";
import { cassandra } from "@lib/cassandra";
import { jsonResponse } from "@lib/http";
import { pika } from "@lib/pika";
import type { Client } from "cassandra-driver";

const routeDef: RouteDef = {
	method: "DELETE",
	accepts: "application/json",
	returns: "application/json;charset=utf-8",
	needsBody: "json",
};

async function handler(
	request: ExtendedRequest,
	requestBody: unknown,
): Promise<Response> {
	const user: UserSession | null = request.session;
	if (!user) {
		return jsonResponse(401, {
			message: "Unauthorized",
			error: "You must be logged in to delete a channel",
		});
	}

	const { id: guildId } = request.params;
	if (!guildId || typeof guildId !== "string") {
		return jsonResponse(400, {
			message: "Missing or invalid guild ID",
		});
	}

	const { id: channel_id } = requestBody as Record<string, unknown>;

	if (typeof channel_id !== "string" || !pika.validate(channel_id, "channel")) {
		return jsonResponse(400, {
			message: "Validation failed",
			error: "Invalid or missing channel_id",
		});
	}

	const client: Client = cassandra.getClient();

	try {
		const result = await client.execute(
			"SELECT id, guild_id FROM channels WHERE id = ?",
			[channel_id],
			{ prepare: true },
		);

		if (result.rowLength === 0) {
			return jsonResponse(404, {
				message: "Channel not found",
				error: "The specified channel does not exist",
			});
		}

		const channel = result.first();

		if (channel.guild_id !== guildId) {
			return jsonResponse(403, {
				message: "Forbidden",
				error: "Channel does not belong to this guild",
			});
		}

		await client.execute("DELETE FROM channels WHERE id = ?", [channel_id], {
			prepare: true,
		});

		logger.custom(
			"[CHANNEL DELETE]",
			`(${channel_id})`,
			`from guild ${guildId} by ${user.id}`,
			"31",
		);

		return jsonResponse(200, {
			message: "Channel deleted successfully",
		});
	} catch (error) {
		logger.error(["Failed to delete channel:", error as Error]);
		return jsonResponse(500, {
			message: "Internal server error",
			error: "Failed to delete channel",
		});
	}
}

export { handler, routeDef };
