import { logger } from "@creations.works/logger";
import { cassandra } from "@lib/cassandra";
import { jsonResponse } from "@lib/http";
import { pika } from "@lib/pika";
import type { Client } from "cassandra-driver";

const routeDef: RouteDef = {
	method: "POST",
	accepts: "application/json",
	returns: "application/json;charset=utf-8",
	needsBody: "json",
};

async function handler(
	request: ExtendedRequest,
	requestBody: unknown,
): Promise<Response> {
	const user: UserSession | null = request.session;
	if (!user) {
		return jsonResponse(401, {
			message: "Unauthorized",
			error: "You must be logged in to move a channel",
		});
	}

	const { id: guildId } = request.params;
	if (!guildId || typeof guildId !== "string") {
		return jsonResponse(400, {
			message: "Missing or invalid guild ID",
		});
	}

	const { channel_id, position, category_id } = requestBody as Record<
		string,
		unknown
	>;

	const errors: string[] = [];

	if (typeof channel_id !== "string" || !pika.validate(channel_id, "channel")) {
		errors.push("Invalid or missing channel_id");
	}

	if (typeof position !== "number" || position < 0) {
		errors.push("Invalid or missing position");
	}

	if (category_id !== undefined) {
		if (
			typeof category_id !== "string" ||
			!pika.validate(category_id, "category")
		) {
			errors.push("Invalid category_id");
		}
	}

	if (errors.length > 0) {
		return jsonResponse(400, {
			message: "Validation failed",
			error: errors.join("; "),
		});
	}

	const client: Client = cassandra.getClient();

	// TODO: check if user has permission to move the channel

	try {
		const existing = await client.execute(
			"SELECT id FROM channels WHERE id = ? AND guild_id = ?",
			[channel_id, guildId],
			{ prepare: true },
		);

		if (existing.rowLength === 0) {
			return jsonResponse(404, {
				message: "Channel not found",
				error: "The specified channel does not exist in this guild",
			});
		}

		if (category_id !== undefined) {
			const catCheck = await client.execute(
				"SELECT id FROM categories WHERE id = ? AND guild_id = ?",
				[category_id, guildId],
				{ prepare: true },
			);

			if (catCheck.rowLength === 0) {
				return jsonResponse(400, {
					message: "Validation failed",
					error: "Provided category does not exist in this guild",
				});
			}
		}

		const now = new Date();

		await client.execute(
			"UPDATE channels SET position = ?, category_id = ?, updated_at = ? WHERE id = ? AND guild_id = ?",
			[position, category_id ?? null, now, channel_id, guildId],
			{ prepare: true },
		);

		logger.custom(
			"[CHANNEL MOVE]",
			`(${channel_id})`,
			`moved to pos ${position}, cat ${category_id ?? "null"} in guild ${guildId} by ${user.id}`,
			"33",
		);

		return jsonResponse(200, {
			message: "Channel moved successfully",
			data: {
				id: channel_id,
				guildId,
				position,
				category_id: category_id ?? null,
				updated_at: now,
			},
		});
	} catch (error) {
		logger.error(["Failed to move channel:", error as Error]);
		return jsonResponse(500, {
			message: "Internal server error",
			error: "Failed to move channel",
		});
	}
}

export { handler, routeDef };
